declare class GenerateArray {
    private static _characters;
    private static _charactersWithSpecial;
    /**
     * Generate an array of the specified length filled with undefined values
     *
     * Example:
     * GenerateArray.blank(5) -> [undefined, undefined, undefined, undefined, undefined]
     *
     * @param length Size of array
     */
    static blank(length: number): any[];
    /**
     * Generate an array of the specified length filled with the specified value
     *
     * Example:
     * GenerateArray.uniform(5, 7) -> [7, 7, 7, 7, 7]
     *
     * @param length Size of array
     * @param value Value to fill array with
     */
    static uniform(length: number, value: any): any[];
    /**
     * Generate an array of the specified length filled with values generated by the given function
     *
     * Examples:
     * GenerateArray.custom(() => Math.random(), 5) -> [0.12345, 0.6789, 0.101112, 0.131415, 0.161718] (possible values)
     * GenerateArray.custom(() => Math.floor(Math.random() * 10), 6) -> [7, 2, 5, 9, 1, 3] (possible values)
     *
     * @param generator Function to generate each array value
     * @param length Size of array
     */
    static custom: (generator: () => any, length: number) => any[];
    /**
     * Generate an array of numbers from the start value up to a max/min value (both inclusive) with the given step value
     *
     * Examples:
     * GenerateArray.counting(0, 5) -> [0, 1, 2, 3, 4, 5]
     * GenerateArray.counting(1, 10, 2) -> [1, 3, 5, 7, 9]
     * GenerateArray.counting(1.2, 37.9, 4.7) -> [1.2, 5.9, 10.6, 15.3, 20, 24.7, 29.4, 34.1]
     * GenerateArray.counting(10, 1, 2, true) -> [10, 8, 6, 4, 2]
     *
     * @param start Number to start at
     * @param end Value to stop at (may not be included in resulting array)
     * @param step Value added at each step (default 1). Always positive - is subtract is 2, this number is just subtracted
     * @param subtract If true, subtracts the step each time
     */
    static counting: (start: number, end: number, step?: number, subtract?: boolean) => number[];
    /**
     * Generate an array of random integers of the specified length within the given range
     *
     * Examples:
     * GenerateArray.integers(6) -> [23, 59, 2, 91, 45, 21] (possible values)
     * GenerateArray.integers(5, 10, 20) -> [15, 17, 12, 19, 11] (possible values)
     *
     * @param length Size of array (>= 1)
     * @param min Minimum value (inclusive), default 0
     * @param max Maximum value (exclusive), default 100
     */
    static integers: (length: number, min?: number, max?: number) => number[];
    /**
     * Generate an array of random decimals of the specified length within the given range. May sometimes produce
     * integers if the range is significantly large
     *
     * Examples:
     * GenerateArray.decimals(5) -> [0.12345, 0.6789, 0.101112, 0.131415, 0.161718] (possible values)
     * GenerateArray.decimals(6, 0, 10) -> [7.12345, 2.6789, 5.101112, 9.131415, 1.161718, 3.192021] (possible values)
     *
     * @param length Size of array (>= 1)
     * @param min Minimum value (inclusive), default 0
     * @param max Maximum value (exclusive), default 1
     */
    static decimals: (length: number, min?: number, max?: number) => number[];
    /**
     * Generate an array of random strings of the specified length within the given range
     *
     * @param length Size of array (>= 1)
     * @param minLength Minimum length of string (>= 1), default 1
     * @param maxLength Maximum length of string (>= minLength), default 10
     * @param specialChars If true, includes special characters (!@#$%^&*()_+/\{}[]|;:'",.<>?`~) in the generated
     * strings. By default, only letters and numbers are used (a-zA-Z0-9)
     */
    static strings: (length: number, minLength?: number, maxLength?: number, specialChars?: boolean) => string[];
    /**
     * Generate an array of the specified length. Each value is obtained from a random generator with equal chances
     *
     * Example:
     * CustomArray.random(5, [() => Math.random(), () => Math.floor(Math.random() * 10)]) -> [0.12345, 7, 0.101112, 1, 3] (possible values)
     *
     * @param length Size of array to be generated (>= 1)
     * @param generators Array of functions that generate a value (value can be anything)
     */
    static generators: (length: number, generators: (() => any)[]) => any[];
    /**
     * Generate array of the specified length with the specified generators. Each generator has a weighted chance of
     * being used for each element in the array
     *
     * Example:
     * CustomArray.randomChance(7, [{ generator: () => Math.random(), chance: 0.3 }, { generator: () => Math.floor(Math.random() * 10), chance: 0.7 }]) -> [4, 0.12345, 7, 0.101112, 7, 1, 8] (possible values)
     *
     * @param length Size of array to be generated (>= 1)
     * @param generators Array of objects in the form { generator: () => any, chance: number }. Chance for all
     * generators combined must add up to 1
     */
    static weightedGenerators: (length: number, generators: {
        generator: () => any;
        chance: number;
    }[]) => any[];
    /**
     * Generates an array of the specified length with the specified generators. Each generator is called a fixed
     * amount of times
     *
     * @param length Size of array to be generated (>= 1)
     * @param generators Array of objects in the form { generator: () => any, count: number }. Generators must
     * return a value (any value), and the count of all generators combined must equal the length of the array
     * @param random If true (default), the order of the generated values is random. If false, all values from a
     * specific generator are next to each other in a sequence
     */
    static fixedCountGenerators: (length: number, generators: {
        generator: () => any;
        count: number;
    }[], random?: boolean) => any[];
    private static _deepArray;
    /**
     * Generates an multidimensional array of the specific length (number of arrays in each array) and depth (how many
     * levels of arrays to generate before stopping) where the base array is empty.
     *
     * Examples:
     * GenerateArray.emptyND(1, 2) -> [[]] (Each array has 1 array in it, and there are 2 levels of arrays)
     * GenerateArray.emptyND(2, 2) -> [[], []] (Each array has 2 arrays in it, and there are 2 levels of arrays)
     * GenerateArray.emptyND(1, 3) -> [[[]]] (There is 1 array in each array, and 3 levels of arrays)
     * GenerateArray.emptyND(2, 3) -> [[[], []], [[], []]] (There are 2 arrays in each array, and 3 levels of arrays)
     *
     * @param length Number of arrays in each array (>= 1), excluding the base array (array at the lowest depth)
     * @param depth Dimension of the returned array (>= 2), i.e. how many layers of arrays there are. Dept of 1 is
     * just [] regardless of length, so not allowed to reduce confusion
     */
    static emptyND: (length: number, depth: number) => any[];
    /**
     * Generates a multidimensional array of the specific length (number of arrays in each array) and depth (how many
     * levels of arrays to generate before stopping) where all base arrays are filled with the given value(s).
     *
     * If the value is an array, the base arrays will be filled with the values of the array instead of adding an
     * extra depth of arrays, e.g. 7 -> [7], [1, 2, 3] -> [1, 2, 3] (put the array in an array if you'd like this)
     *
     * Examples:
     * GenerateArray.uniformND(7, 3, 2) -> [[7], [7], [7]]
     * GenerateArray.uniformND([1, 2, 3], 3, 2) -> [[1, 2, 3], [1, 2, 3], [1, 2, 3]]
     *
     * @param value Value of the base array. If not an array, the base array will be an array filled with this value.
     * @param length Number of arrays in each array (>= 1), excluding the base array (array at the lowest depth)
     * @param depth Dimension of the returned array (>= 2), i.e. how many layers of arrays there are. Depth of 1
     * isn't included to reduce confusion, as it would just return the value in an array (or the value if it's an array)
     * e.g. MultidimensionalArray.uniform([1, 2, 3], 3, 1) -> [1, 2, 3]
     */
    static uniformND: (value: any, length: number, depth: number) => any[];
    /**
     * Generates a multidimensional array of the specific length (number of arrays in each array) and depth (how many
     * levels of arrays to generate before stopping) where the base arrays are generated by the given function.
     *
     * Examples:
     * GenerateArray.customND(() => Math.floor(Math.random() * 10), 3, 2) -> [[7], [2], [5]] (possible output)
     *
     * @param generator Function to generate the base array. Note this function is called once for each base array,
     * so to have multiple values, this function should return an array of multiple values (e.g.
     * GenerateArray.custom(() => [1, 2, 3], len, depth))
     * @param length Number of arrays in each array (>= 1), excluding the base array (array at the lowest depth)
     * @param depth Dimension of the returned array (>= 2), i.e. how many layers of arrays there are. Depth of 1 isn't
     * allowed to reduce confusion as it would just put the value of one function call in an array, e.g. () => 7 -> [7]
     * (to get a 1-D array with multiple values, use GenerateArray.custom(() => ..., len) instead)
     */
    static customND: (generator: () => any[], length: number, depth: number) => any[];
}

declare class ArrayUtils {
    /**
     * Adds the specified number of dimensions to the array (i.e. wraps the array in arrays the specified number of times)
     *
     * Example:
     * addDimensions(arr, 1) -> [arr]
     * addDimensions(arr, 2) -> [[arr]]
     * addDimensions(arr, 4) -> [[[[arr]]]]
     *
     * @param arr Array to add dimensions to
     * @param depth Number of dimensions to add (>= 1)
     */
    static addDimensions<T>(arr: T[], depth: number): T[];
    /**
     * Flattens the array to a single dimension. This removes all nested arrays, placing all non-array elemnts in a single array
     *
     * Example:
     * flatten([1, [2, 3], [4, [5, 6]]]) -> [1, 2, 3, 4, 5, 6]
     *
     * @param arr
     */
    static flatten(arr: any[]): any[];
    /**
     * Multiplies the array within itself n times (i.e. repeats the elements in the array n times). Does not modify the original array
     *
     * Example:
     * multiplyLength([1, 2], 3) -> [1, 2, 1, 2, 1, 2]
     * multiplyLength([1, 2], 3, true) -> [1, 1, 1, 2, 2, 2]
     *
     * @param arr Array to multiply
     * @param factor Number of times to multiply the array (>= 2)
     * @param elementWise If false (default), repeats the entire array before repeating the next array. If true,
     * repeats each element n times before repeating the next element in the array
     */
    static multiplyLength(arr: any[], factor: number, elementWise?: boolean): any[];
    /**
     * Returns a copy of the given array with any duplicate values removed
     *
     * @param arr Array to remove duplicates from
     */
    static removeDuplicates(arr: any[]): any[];
}

declare class GenerateArrayError extends Error {
    constructor(message: string);
}

export { ArrayUtils, GenerateArray, GenerateArrayError };
